# coding: utf-8

TAG_FUNCTION = 'func'
TAG_MEMORY = 'memory'
TAG_SYS_MEMORY = 'sys_memory'
TAG_BATTERY = 'battery'
TAG_CPU = 'cpu'
TAG_SNAPSHOT = 'screen'

AIRNATIVE = '/data/local/tmp/air-native'

ROTATION_0 = 0
ROTATION_90 = 1
ROTATION_180 = 2
ROTATION_270 = 3

ENV_SERIALNO = 'AIRTEST_SERIALNO'